unit uOptionsUI;
{$INCLUDE 'Plugin.inc'}

interface

uses
  Windows, SysUtils, Classes,
  {$INCLUDE 'DelphiFarUnits.inc'} ,
  uPluginConsts, uMessages, uOptions;

type
  TOptionsDialog = class(TOkCancelDialog)
  private
    fOptions: TOptions;
  private
    fCheckSaveLastPanelDirectory: TFarDialog.TCheckbox;
    fCheckUseSubstEngines: TFarDialog.TCheckbox;
    fCheckUseSubstEngine_KnownFolders: TFarDialog.TCheckbox;
    fCheckUseSubstEngine_Environment: TFarDialog.TCheckbox;
    fCheckUseSubstEngine_BatchEnvironment: TFarDialog.TCheckbox;
    fCheckUseSubstEngine_Registry: TFarDialog.TCheckbox;
    fCheckAllowPathSuffix: TFarDialog.TCheckbox;
    fCheckAllowPluginCalls: TFarDialog.TCheckbox;
    fCheckAllowFuzzySearch: TFarDialog.TCheckbox;
    fCheckAllowSearchOverDirectories: TFarDialog.TCheckbox;
  protected
    procedure Clear; override;
    procedure CreateUserControls; override;
    procedure SetSize(Width, Height: Integer); override;
    procedure ConsoleSizeToDialogSize(var Width, Height: Integer); override;
    property CheckSaveLastPanelDirectory: TFarDialog.TCheckbox read fCheckSaveLastPanelDirectory;
    property CheckUseSubstEngines: TFarDialog.TCheckbox read fCheckUseSubstEngines;
    property CheckUseSubstEngine_KnownFolders: TFarDialog.TCheckbox read fCheckUseSubstEngine_KnownFolders;
    property CheckUseSubstEngine_Environment: TFarDialog.TCheckbox read fCheckUseSubstEngine_Environment;
    property CheckUseSubstEngine_BatchEnvironment: TFarDialog.TCheckbox read fCheckUseSubstEngine_BatchEnvironment;
    property CheckUseSubstEngine_Registry: TFarDialog.TCheckbox read fCheckUseSubstEngine_Registry;
    property CheckAllowPathSuffix: TFarDialog.TCheckbox read fCheckAllowPathSuffix;
    property CheckAllowPluginCalls: TFarDialog.TCheckbox read fCheckAllowPluginCalls;
    property CheckAllowFuzzySearch: TFarDialog.TCheckbox read fCheckAllowFuzzySearch;
    property CheckAllowSearchOverDirectories: TFarDialog.TCheckbox read fCheckAllowSearchOverDirectories;
  public
    class function ExecuteDialog: boolean;
    constructor Create;
    destructor Destroy; override;
    function Execute: boolean;
    property Options: TOptions read fOptions write fOptions;
  end;

implementation

{ TOptionsDialog }

procedure TOptionsDialog.Clear;
begin
  inherited;
  fCheckSaveLastPanelDirectory := nil;
  fCheckUseSubstEngines := nil;
  fCheckUseSubstEngine_KnownFolders := nil;
  fCheckUseSubstEngine_Environment := nil;
  fCheckUseSubstEngine_BatchEnvironment := nil;
  fCheckUseSubstEngine_Registry := nil;
  fCheckAllowPathSuffix := nil;
  fCheckAllowPluginCalls := nil;
  fCheckAllowFuzzySearch := nil;
  fCheckAllowFuzzySearch := nil;
end;

constructor TOptionsDialog.Create;
begin
  inherited Create(DIALOG_CONFIG_GUID);
end;

procedure TOptionsDialog.CreateUserControls;
begin
  inherited;
  fCheckSaveLastPanelDirectory := AddCheckbox(0, 0, GetMsg(MSaveLastPanelDirectory), False);
  fCheckUseSubstEngines := AddCheckbox(0, 0, GetMsg(MUseSubstEngines), False);
  fCheckUseSubstEngine_KnownFolders := AddCheckbox(0, 0, GetMsg(MUseSubstEngine_KnownFolders), False);
  fCheckUseSubstEngine_Environment := AddCheckbox(0, 0, GetMsg(MUseSubstEngine_Environment), False);
  fCheckUseSubstEngine_BatchEnvironment := AddCheckbox(0, 0, GetMsg(MUseSubstEngine_BatchEnvironment), False);
  fCheckUseSubstEngine_Registry := AddCheckbox(0, 0, GetMsg(MUseSubstEngine_Registry), False);
  fCheckAllowPathSuffix := AddCheckbox(0, 0, GetMsg(MAllowPathSuffix), False);
  fCheckAllowPluginCalls := AddCheckbox(0, 0, GetMsg(MAllowPluginCalls), False);
  fCheckAllowFuzzySearch := AddCheckbox(0, 0, GetMsg(MAllowFuzzySearch), False);
  fCheckAllowSearchOverDirectories := AddCheckbox(0, 0, GetMsg(MAllowSearchOverDirectories), False);
end;

destructor TOptionsDialog.Destroy;
begin
  inherited;
end;

function TOptionsDialog.Execute: boolean;
begin
  Result := False;
  CheckSaveLastPanelDirectory.Checked := Options.SaveLastPanelDirectory;
  CheckUseSubstEngines.Checked := Options.UseSubstitutionEngines;
  CheckUseSubstEngine_KnownFolders.Checked := Options.UseSubstitutionEngine_KnownFolders;
  CheckUseSubstEngine_Environment.Checked := Options.UseSubstitutionEngine_Environment;
  CheckUseSubstEngine_BatchEnvironment.Checked := Options.UseSubstitutionEngine_BatchEnvironment;
  CheckUseSubstEngine_Registry.Checked := Options.UseSubstitutionEngine_Registry;
  CheckAllowPathSuffix.Checked := Options.AllowPathSuffix;
  CheckAllowPluginCalls.Checked := Options.AllowPluginCalls;
  CheckAllowFuzzySearch.Checked := Options.AllowFuzzySearch;
  CheckAllowSearchOverDirectories.Checked := Options.AllowSearchOverDirectories;
  if inherited Execute then
  begin
    Options.SaveLastPanelDirectory := CheckSaveLastPanelDirectory.Checked;
    Options.UseSubstitutionEngines := CheckUseSubstEngines.Checked;
    Options.UseSubstitutionEngine_KnownFolders := CheckUseSubstEngine_KnownFolders.Checked;
    Options.UseSubstitutionEngine_Environment := CheckUseSubstEngine_Environment.Checked;
    Options.UseSubstitutionEngine_BatchEnvironment := CheckUseSubstEngine_BatchEnvironment.Checked;
    Options.UseSubstitutionEngine_Registry := CheckUseSubstEngine_Registry.Checked;
    Options.AllowPathSuffix := CheckAllowPathSuffix.Checked;
    Options.AllowPluginCalls := CheckAllowPluginCalls.Checked;
    Options.AllowFuzzySearch := CheckAllowFuzzySearch.Checked;
    Options.AllowSearchOverDirectories := CheckAllowSearchOverDirectories.Checked;
    Result := True;
  end;
end;

class function TOptionsDialog.ExecuteDialog: boolean;
var
  Options: TOptions;
  Dlg: TOptionsDialog;
begin
  Result := False;
  Options := TOptions.Create;
  try
    Dlg := TOptionsDialog.Create;
    try
      Dlg.Options := Options;
      if Dlg.Execute then
      begin
        Options.Save;
        Result := True;
      end;
    finally
      FreeAndNil(Dlg);
    end;
  finally
    FreeAndNil(Options);
  end;
end;

procedure TOptionsDialog.SetSize(Width, Height: Integer);
begin
  inherited;
  AutoPos(CheckSaveLastPanelDirectory);
  AutoPos(CheckUseSubstEngines);
  AutoPos(CheckUseSubstEngine_KnownFolders, 2);
  AutoPos(CheckUseSubstEngine_Environment, 2);
  AutoPos(CheckUseSubstEngine_BatchEnvironment, 2);
  AutoPos(CheckUseSubstEngine_Registry, 2);
  AutoPos(CheckAllowPathSuffix);
  AutoPos(CheckAllowPluginCalls);
  AutoPos(CheckAllowFuzzySearch);
  AutoPos(CheckAllowSearchOverDirectories);
end;

procedure TOptionsDialog.ConsoleSizeToDialogSize(var Width, Height: Integer);
begin
  inherited;
  Height := 16;
end;

end.
