unit uPlugin;
{$INCLUDE 'Plugin.inc'}

interface

uses
  Windows, SysUtils, Classes,
  {$IFDEF DEBUG_LOGS} uLog. {$ENDIF}
  {$INCLUDE 'DelphiFarUnits.inc'} ;

implementation

uses
  uSystem, uPluginConsts, uMessages, uOptions, uOptionsUI, uMakeDirUI, uTasks, uPanel;

type
  TPlugin = class(TFarPlugin)
  private
  protected
    function PluginID: TGUID; override;
    function GetPluginTitle(out Description, Author: string): string; override;
    procedure GetPluginInfo(var Flags: TPluginFlags; var MainMenu, ConfigMenu, DiskMenu: TFarPlugin.TFarPluginMenu; var CommandPrefix: string); override;
    function GetCommonString(ID: TFarPlugin.TCommonString): string; override;
    function Execute(OpenFrom: DWORD; GUID: PGUID; Data: TIntPtr): THandle; override;
    procedure DoLog(const Msg: string); override;
    function PluginConfig(GUID: PGUID; MenuItem: TIntPtr): boolean; override;
  public
    constructor Create;
    destructor Destroy; override;
  end;

const
  PLUGIN_TITLE = 'Named Folders Lite';
  PLUGIN_DESCRIPTION = 'Named Folders Lite';
  PLUGIN_AUTHOR = 'Pepak, https://www.pepak.net';

{ TPlugin }

constructor TPlugin.Create;
begin
  inherited;
end;

destructor TPlugin.Destroy;
begin
  inherited;
end;

procedure TPlugin.DoLog(const Msg: string);
begin
  {$IFDEF DEBUG_LOGS}
  uLog.Log(Msg);
  {$ENDIF}
end;

function TPlugin.Execute(OpenFrom: DWORD; GUID: PGUID; Data: TIntPtr): THandle;
var
  CommandLine: string;
begin
  {$IFDEF DEBUG_LOGS}
  Log('TPlugin.Execute START');
  {$ENDIF}
  {$IFDEF DEBUG_LOGS}
  Log('OpenFrom = %d', [OpenFrom]);
  {$ENDIF}
  Result := 0;
  if OpenFrom = OPEN_COMMANDLINE then
  begin
    CommandLine := CopyString( {$IFDEF FAR3_UP} POpenCommandLineInfo(Data)^.CommandLine {$ELSE} PFarChar(Data) {$ENDIF} );
    {$IFDEF DEBUG_LOGS}
    Log('Command line = %s', [CommandLine]);
    {$ENDIF}
    if CommandLine = '' then
    begin
      {$IFDEF DEBUG_LOGS}
      Log('Opening panel from command line');
      {$ENDIF}
      Result := TNamedFolders.OpenPanel.Handle;
      {$IFDEF DEBUG_LOGS}
      Log('Result = 0x%08.8x', [Result]);
      {$ENDIF}
    end
    else
    begin
      TNamedFolders.ProcessCommandLine(CommandLine);
    end;
  end
  {$IFDEF FAR3_UP}
  else if OpenFrom = OPEN_FROMMACRO_ then
  begin
    {$IFDEF DEBUG_LOGS}
    Log('Opening from macro');
    {$ENDIF}
    Result := THandle(TNamedFolders.HandleMacroCall(POpenMacroInfo(Data)));
    {$IFDEF DEBUG_LOGS}
    Log('Result = %d', [Result]);
    {$ENDIF}
  end
  {$ENDIF}
  else if OpenFrom = OPEN_SHORTCUT then
  begin
    {$IFDEF DEBUG_LOGS}
    Log('Opening from shortcut');
    {$ENDIF}
    Result := TNamedFolders.OpenPanel.Handle;
    {$IFDEF DEBUG_LOGS}
    Log('Result = 0x%08.8x', [Result]);
    {$ENDIF}
  end
  else if OpenFrom = OPEN_DISKMENU then
  begin
    {$IFDEF DEBUG_LOGS}
    Log('Opening in left panel');
    {$ENDIF}
    Result := TNamedFolders.OpenPanel.Handle;
    {$IFDEF DEBUG_LOGS}
    Log('Result = 0x%08.8x', [Result]);
    {$ENDIF}
  end
  else if {$IFDEF FAR3_UP} OpenFrom = OPEN_RIGHTDISKMENU {$ELSE} False {$ENDIF} then
  begin
    {$IFDEF DEBUG_LOGS}
    Log('Opening in right panel');
    {$ENDIF}
    Result := TNamedFolders.OpenPanel.Handle;
    {$IFDEF DEBUG_LOGS}
    Log('Result = 0x%08.8x', [Result]);
    {$ENDIF}
  end
  else if OpenFrom = OPEN_PLUGINSMENU then
  begin
    {$IFDEF DEBUG_LOGS}
    Log('Opening from plugin menu');
    {$ENDIF}
    Result := TNamedFolders.OpenPanel.Handle;
    {$IFDEF DEBUG_LOGS}
    Log('Result = 0x%08.8x', [Result]);
    {$ENDIF}
  end;
  {$IFDEF DEBUG_LOGS}
  Log('TPlugin.Execute END');
  {$ENDIF}
end;

function TPlugin.GetCommonString(ID: TFarPlugin.TCommonString): string;
begin
 case ID of
    TFarPlugin.TCommonString.MTitle:
      Result := GetMsg(MTitle);
    TFarPlugin.TCommonString.MExceptionOccurredIn:
      Result := GetMsg(MExceptionOccurredIn);
    TFarPlugin.TCommonString.MOk:
      Result := GetMsg(MOk);
    TFarPlugin.TCommonString.MCancel:
      Result := GetMsg(MCancel);
    TFarPlugin.TCommonString.MYes:
      Result := GetMsg(MYes);
    TFarPlugin.TCommonString.MNo:
      Result := GetMsg(MNo);
    else
      Result := inherited GetCommonString(ID);
  end;
end;

procedure TPlugin.GetPluginInfo(var Flags: TPluginFlags; var MainMenu, ConfigMenu, DiskMenu: TFarPlugin.TFarPluginMenu; var CommandPrefix: string);
begin
  SetLength(MainMenu, 1);
  MainMenu[0].Title := GetMsg(MTitle);
  MainMenu[0].ID := MENU_DISK_GUID;
  SetLength(ConfigMenu, 1);
  ConfigMenu[0].Title := GetMsg(MTitle);
  ConfigMenu[0].ID := MENU_CONFIG_GUID;
  DiskMenu := MainMenu;
  CommandPrefix := 'cd';
end;

function TPlugin.GetPluginTitle(out Description, Author: string): string;
begin
  Result := PLUGIN_TITLE;
  Description := PLUGIN_DESCRIPTION;
  Author := PLUGIN_AUTHOR;
end;

function TPlugin.PluginConfig(GUID: PGUID; MenuItem: TIntPtr): boolean;
begin
  Result := False;
  //if Info.GUID^ = MENU_CONFIG_GUID then begin
    if TOptionsDialog.ExecuteDialog then
      Result := True;
  //end;
end;

function TPlugin.PluginID: TGUID;
begin
  Result := PLUGIN_GUID;
end;

exports
  {$IFDEF FAR3_UP}
  GetOpenPanelInfoW,
  ClosePanelW,
  GetFindDataW,
  FreeFindDataW,
  MakeDirectoryW,
  SetDirectoryW,
  DeleteFilesW,
  GetFilesW,
  PutFilesW,
  ProcessPanelInputW,
  ProcessPanelEventW;
  {$ELSE}
  ClosePluginW,
  GetFindDataW,
  FreeFindDataW,
  MakeDirectoryW,
  SetDirectoryW,
  DeleteFilesW,
  GetFilesW,
  PutFilesW,
  ProcessEventW,
  ProcessKeyW;
  {$ENDIF}

initialization
  TFarPlugin.InitInstance(TPlugin.Create);

end.
